package com.mediahub

import akka.NotUsed
import akka.actor.ActorSystem
import akka.stream.alpakka.csv.scaladsl.{CsvParsing, CsvToMap}
import akka.stream.scaladsl.{Compression, FileIO, Flow, Sink, Source}
import akka.util.ByteString
import org.springframework.stereotype.Component

import java.io.File
import java.nio.file.Paths
import scala.collection.immutable
import scala.concurrent.duration.DurationInt
import scala.concurrent.{Await, Future}
import scala.language.postfixOps


@Component
class FileReader {

  implicit val system: ActorSystem = ActorSystem()

  val titleBasicsResource: File = new File("src/main/resources/title.basics.tsv.gz")
  val titlePrincipalsResource: File = new File("src/main/resources/title.principals.tsv.gz")
  val nameBasicsResource: File = new File("src/main/resources/name.basics.tsv.gz")

  def fileSource(file: File) = {
    FileIO
      .fromPath(Paths.get(file.getPath), 1 * 1024 * 1024)
      .via(Compression.gunzip())
  }

  val lineParser: Flow[ByteString, Map[String, String], NotUsed] = {
    CsvParsing
      .lineScanner(CsvParsing.Tab, CsvParsing.DoubleQuote, CsvParsing.DoubleQuote)
      .via(CsvToMap.toMapAsStringsCombineAll(headerPlaceholder = Option.empty))
  }

  //working search not optimized
  val titleId: String = getIdOfTitle(titleBasicsResource, "Carmencita")
  println("value ", titleId)

  val personsIdList: List[String] = getIdOfPersons(titlePrincipalsResource, titleId)
  println("value ", personsIdList)

  val personsList: List[String] = getPersons(nameBasicsResource, personsIdList)
  println("value ", personsList)


  //working do not change for now
  def getIdOfTitle(file: File, titleName: String): String = {
    val result: Future[Option[String]] = Source.single(file)
      .flatMapConcat(f => FileIO.fromPath(Paths.get(f.getPath), 1 * 1024 * 1024).reduce((a, b) => a ++ b))
      .via(Compression.gunzip())
      .via(CsvParsing.lineScanner(CsvParsing.Tab, CsvParsing.DoubleQuote, CsvParsing.DoubleQuote))
      .via(CsvToMap.toMapAsStringsCombineAll(headerPlaceholder = Option.empty))
      .filter(row => row.getOrElse("primaryTitle", "") == titleName)
      .map(a => a.get("tconst"))
      .runWith(Sink.head)

    Await.result(result, 5 minutes)
    result.value.get.get.get
  }

  def getIdOfTitleAsync(file: File, titleName: String): immutable.Iterable[Option[String]] = {
    val result = fileSource(file)
      .mapAsync(10) {
        result =>
          Source.single(result)
            .via(lineParser)
            .filter(row => row.getOrElse("primaryTitle", "") == titleName)
            .map(a => a.get("tconst"))
            .runWith(Sink.collection)
      }
      .runWith(Sink.head)
    Await.result(result, 5 minutes)
  }

  def getIdOfPersons(file: File, titleId: String): List[String] = {
    val result: Future[immutable.Iterable[Option[String]]] = Source.single(file)
      .flatMapConcat(f => FileIO.fromPath(Paths.get(f.getPath), 1 * 1024 * 1024))
      .via(Compression.gunzip())
      .via(CsvParsing.lineScanner(CsvParsing.Tab, CsvParsing.DoubleQuote, CsvParsing.DoubleQuote))
      .via(CsvToMap.toMapAsStringsCombineAll(headerPlaceholder = Option.empty))
      .filter(row => row.getOrElse("tconst", "") == titleId)
      .map(a => a.get("nconst"))
      .runWith(Sink.collection)

    Await.result(result, 5 minutes)
    result.value.get.get.toList.flatten
  }

  def getPersons(file: File, personsIdList: List[String]): List[String] = {
    val result: Future[immutable.Iterable[Option[String]]] = Source.single(file)
      .flatMapConcat(f => FileIO.fromPath(Paths.get(f.getPath), 1 * 1024 * 1024))
      .via(Compression.gunzip())
      .via(CsvParsing.lineScanner(CsvParsing.Tab, CsvParsing.DoubleQuote, CsvParsing.DoubleQuote))
      .via(CsvToMap.toMapAsStringsCombineAll(headerPlaceholder = Option.empty))
      .filter(row => personsIdList.contains(row.getOrElse("nconst", "")))
      .map(a => a.get("primaryName"))
      .runWith(Sink.collection)

    Await.result(result, 5 minutes)
    result.value.get.get.toList.flatten
  }
}
